import java.util.*;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.nio.channels.FileChannel;
import java.nio.charset.StandardCharsets;

public class MyRandomAccess {
    public static void main(String... args) {
        if (args.length <= 0) {
            System.out.println("コマンドライン引数でファイル名を指定してください");
            System.exit(0);
        }
        try (FileChannel file = FileChannel.open(Paths.get(args[0]), 
                EnumSet.of(StandardOpenOption.READ, StandardOpenOption.WRITE))) {
            file.position(file.size());     // ファイルの末尾へ移動
            file.write(ByteBuffer.wrap("zzz".getBytes())); // ファイルの末尾へ書き込み

            file.position(0);               // ファイルの先頭へ移動

            // 以下、ファイルの中身をすべて読み込む処理
            ByteBuffer buf = ByteBuffer.allocate(4096);
            int len;
            while ((len = file.read(buf)) != -1) {  // ファイルからByteBufferへ読み込み
                buf.flip();                         // ByteBufferの有効データ領域をセット
                System.out.print(StandardCharsets.UTF_8.decode(buf)); // ByteBufferのバイト列を文字列にして標準出力
                buf.clear();                        // ByteBufferの中身をクリア
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
