import java.io.IOException;
import java.nio.file.*;
import static java.nio.file.StandardWatchEventKinds.*;

public class DirWatch {
    public static void main(String... args) throws IOException, InterruptedException {
        if (args.length <= 0) {
            System.out.println("コマンドライン引数でディレクトリ名を指定してください");
            System.exit(0);
        }

        Path path = Paths.get(args[0]);

        try (WatchService watchService = FileSystems.getDefault().newWatchService()) {
            // ENTRY_CREATEなどはStandardWatchEventKindsの定数
            path.register(watchService, ENTRY_CREATE, ENTRY_DELETE, ENTRY_MODIFY);

            while (true) {
                WatchKey key = watchService.take(); // 変化があるまで呼び出しが停止する（takeの代わりにpollだと停止しない）

                for (WatchEvent<?> event: key.pollEvents()) {
                    System.out.println(event.kind()); //=> StandardWatchEventKindsの定数
                    System.out.println((Path)event.context()); //=> 変更のあったファイルパス
                }
                boolean valid = key.reset();
                if (!valid) {
                    break;
                }
            }
        }
    }
}
