import java.util.concurrent.*;

public class Main {
    private static class Worker {
        public String exec() {
            emulateLongTask();
            return "RESULT";
        }
        public Integer aggregate(String input) {
            return input.length();
        }
        public void showResult(Integer result, Throwable error) {
            if (error != null) {
                System.out.println("error");
                return;
            }
            System.out.println("ret=" + result);
        }
        private void emulateLongTask() { // ネットワークアクセスなど重い処理を想定
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {}
        }
    }

    public static void main(String... args) throws InterruptedException, ExecutionException {
        Worker worker = new Worker();
        CompletableFuture<Integer> future = CompletableFuture.supplyAsync(worker::exec)
            .thenComposeAsync(s -> CompletableFuture.supplyAsync(worker::exec))
            .thenApplyAsync(worker::aggregate)      //=> worker::execの実行結果"RESULT"を入力としてworker::aggregate呼び出し
            .whenCompleteAsync(worker::showResult); //=> "RESULT"の文字列長の6を表示
        future.join();  // 上記のthen処理の終了をここで待つ
    }
}
