import java.util.*;
import java.io.*;
import java.net.*;
import java.util.concurrent.*;

public class MyConcurrentServer {
    private static final int LISTEN_PORT = 9000; // 待ち受けポート番号（クライアントと事前取り決め）
    private static final char END_MARK = '.'; // 終端文字列（クライアントとの事前取り決め）
    private final ExecutorService executor;

    public MyConcurrentServer() {
        this.executor = Executors.newFixedThreadPool(8);
    }

    private static class Worker implements Runnable {
        private final Socket sock;

        Worker(Socket sock) {
            this.sock = sock;
        }

        @Override
        public void run() { // クライアントとの通信処理（1つのクライアントごとに1つのスレッドが対応）
            try (Socket sock = this.sock;  // close処理をtry-with-resources文に任せるための技巧
                 Reader in = new BufferedReader(new InputStreamReader(sock.getInputStream()));
                 Writer out = new BufferedWriter(new OutputStreamWriter(sock.getOutputStream()))) {
                StringBuilder sb = new StringBuilder(4096);

                int c;
                while ((c = in.read()) != -1) {
                    System.out.print(new Character((char)c));
                    if (c == END_MARK) {
                        break;
                    }
                    sb.append((char)c);
                }
                out.write(sb.toString());
                out.flush();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    public void runLoop() {
        try (ServerSocket ssock = new ServerSocket(LISTEN_PORT)) { // 待ち受けソケット（受動的ソケット）
            while (true) {
                Socket sock = ssock.accept(); // クライアントからの接続待ち
                executor.execute(new Worker(sock)); // 受け入れソケットをWorkerに渡してスレッド生成（スレッドプールから取得）
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            executor.shutdown();
        }
    }

    public static void main(String... args) {
        MyConcurrentServer server = new MyConcurrentServer();
        server.runLoop();
    }
}
