import java.util.*;
import java.util.concurrent.*;
import java.io.*;

public class MyProduceConsume {
    private static class Producer implements Runnable {
        private final Queue<String> queue;

        Producer(Queue<String> queue) {
            this.queue = queue;
        }

        @Override
        public void run() { // 通知スレッドのエントリポイント
            Console console = System.console();
            while (true) {
                System.out.println("input any string");
                String s = console.readLine();
                synchronized(queue) {
                    queue.add(s);  // 待機を抜ける条件を成立
                    queue.notifyAll(); // 待機スレッドを起こす
                }
            }
        }
    }

    private static class Consumer implements Runnable {
        private final Queue<String> queue;

        Consumer(Queue<String> queue) {
            this.queue = queue;
        }

        @Override
        public void run() { // 待機スレッドのエントリポイント
            try {
                while (true) {
                    synchronized(queue) {
                        while (queue.isEmpty()) { // 待機を抜ける条件
                            System.out.println("waiting...");
                            queue.wait();         // ここで待機
                        }
                        String s = queue.remove();
                        System.out.println(s + " is consumed");
                    }
                }
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }

    public static void main(String... args) throws InterruptedException, ExecutionException {
        Queue<String> queue = new LinkedList<>(); // 共有データ兼通知オブジェクト

        ExecutorService executor = Executors.newFixedThreadPool(8);

        Future<?> producer = executor.submit(new Producer(queue));
        Future<?> consumer = executor.submit(new Consumer(queue));

        // Futureのgetメソッドでサブスレッド完了を待機
        producer.get();
        consumer.get();
        executor.shutdown();
    }
}
