import java.util.concurrent.*;

public class Main {
    private static class Worker implements Runnable {
        private final CompletableFuture<String> future;        
        public Worker(CompletableFuture<String> future) {
            this.future = future;
        }
        
        @Override
        public void run() {
            emulateLongTask();
            future.complete("RETVAL"); // スレッドから結果を返す
        }

        private void emulateLongTask() {
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {}
        }
    }

    public static void main(String... args) {
        ExecutorService executor = Executors.newFixedThreadPool(8);
        CompletableFuture<String> future = new CompletableFuture<>();
        // スレッドの実行結果で呼ばれるコールバック処理。エラー処理は省略
        future.whenCompleteAsync((result, error) -> { System.out.println("ret = " + result); }); //=> "ret = RETVAL"を表示
        executor.execute(new Worker(future));
        executor.shutdown();
    }
}
