import java.lang.reflect.*;
import java.io.*;
import java.net.*;
import java.util.*;
import javax.tools.JavaCompiler;
import javax.tools.ToolProvider;

public class MyCompiler {
    public static void main(String... args) throws Exception {
        if (args.length < 2) {
            System.err.println("usage: java " + MyCompiler.class.getSimpleName() + " クラス名 メソッド名");
            System.exit(-1);
        }
        String targetClass = args[0];
        String targetMethod = args[1];

        JavaCompiler compiler = ToolProvider.getSystemJavaCompiler();
        if (compiler == null) {
            System.out.println("Compiler is not found");
            return;
        }

        Console con = System.console();
        while (true) {
            // 引数で指定したクラスのソースファイルをコンパイル
            int ret = compiler.run(null, null, null, new String[] { targetClass + ".java" } );
            if (ret == 0) {
                // クラスローダ生成
                ClassLoader loader = URLClassLoader.newInstance(new URL[]{ new File(".").toURI().toURL() }, null);
                // 指定したクラスローダでクラスを再ロード
                Class<?> clazz = Class.forName(targetClass, true, loader);
                if (clazz != null) {
                    Method method = clazz.getMethod(targetMethod);
                    if (method != null) {
                        method.invoke(clazz.newInstance());
                    }
                }
            } else {
                System.out.println("Compile failed");
                break;
            }
            con.printf("Press enter key to continue");
            con.readLine();
        }
    }
}
