import java.util.*;
import java.io.*;
import java.lang.reflect.*;

public class MyEcho3 {
    private static class Filter {
        Filter(Object obj, Method method) {
            this.obj = obj;
            this.method = method;
        }
        final Object obj;
        final Method method;
    }

    private final List<Filter> filters = new ArrayList<>();

    public MyEcho3(String... classNames) throws ReflectiveOperationException {
        // プラグインコードのロード
        for (String className : classNames) {
            Class clazz = Class.forName(className); // リフレクションで探す
            for (Method method : clazz.getMethods()) {
                if (method.isAnnotationPresent(MyFilter.class)) { // アノテーションを探す
                    filters.add(new Filter(clazz.newInstance(), method));
                }
            }
        }
    }

    public void exec() {
        try (BufferedReader stdin = new BufferedReader(new InputStreamReader(System.in))) {
            while (true) {
                System.out.println("Input any text");
                String msg = stdin.readLine();

                String output = msg;
                for (Filter filter : filters) {
                    output = (String)filter.method.invoke(filter.obj, output);
                }
                System.out.println("You said, " + output);
            }
        } catch (IOException | ReflectiveOperationException e) {
            e.printStackTrace();
        }
    }

    public static void main(String... args) throws ReflectiveOperationException {
        MyEcho3 echo = new MyEcho3(args); // コマンドライン引数でクラス名を指定
        echo.exec();
    }
}
