import javafx.application.Application;
import javafx.application.Platform;
import javafx.beans.property.*;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.ProgressBar;
import javafx.scene.layout.HBox;
import javafx.stage.Stage;

public class MyApplication extends Application {
    @Override
    public void start(Stage primaryStage) throws Exception {
        ProgressBar progressBar = new ProgressBar(0.0);
        Button runButton = new Button("Run");
        Button cancelButton = new Button("Cancel");
        cancelButton.setDisable(true);
        
        class MyTask implements Runnable {
            private boolean cancelled;
            public void cancel() {
                cancelled = true;
            }
            public boolean isCancelled() {
                return cancelled;
            }
            
            private DoubleProperty progress =
                    new SimpleDoubleProperty(this, "progress", 0.0);
            public ReadOnlyDoubleProperty progressProperty() {
                return progress;
            }
            public double getProgress() {
                return progress.get();
            }
            private void setProgress(double value) {
                progress.set(value);
            }
            
            private BooleanProperty running = 
                    new SimpleBooleanProperty(this, "running", false);
            public ReadOnlyBooleanProperty runningProperty() {
                return running;
            }
            public boolean isRunning() {
                return running.get();
            }
            private void setRunning(boolean value){
                running.set(value);
            }
            
            @Override
            public void run() {
                Platform.runLater(new Runnable() {
                    @Override
                    public void run() {
                        setRunning(true);
                    }
                });
                for (int i = 0; i <= 100; i++) {
                    if (isCancelled()) {
                        break;
                    }
                    final double p = i/100d;
                    Platform.runLater(new Runnable() {
                        @Override
                        public void run() {
                            setProgress(p);
                        }
                    });
                    try {
                        Thread.sleep(10);
                    } catch (InterruptedException e) {
                    }
                }
                Platform.runLater(new Runnable() {
                    @Override
                    public void run() {
                        setRunning(false);
                    }
                });
            }
        }
        final MyTask[] currentTask = new MyTask[1];
        
        runButton.setOnAction(new EventHandler<ActionEvent>() {
            @Override
            public void handle(ActionEvent event) {
                runButton.setDisable(true); 
                cancelButton.setDisable(false);
                
                MyTask task = new MyTask();
                currentTask[0] = task;
                task.runningProperty().addListener(new ChangeListener<Boolean>() {
                    @Override
                    public void changed(ObservableValue<? extends Boolean> observable,
                                        Boolean oldValue, Boolean newValue) {
                        if (!newValue) {
                            cancelButton.setDisable(true);    
                            runButton.setDisable(false);
                        }
                    }
                }); 
                task.progressProperty().addListener(new ChangeListener<Number>() {
                    @Override
                    public void changed(ObservableValue<? extends Number> observable, Number oldValue, Number newValue) {
                        progressBar.setProgress(newValue.doubleValue());
                    }
                });
                
                Thread thread = new Thread(task);
                thread.setDaemon(true);
                thread.start();
            }
        });
        cancelButton.setOnAction(new EventHandler<ActionEvent>() {
            @Override
            public void handle(ActionEvent event) {
                currentTask[0].cancel();
            }
        });
        
        HBox root = new HBox(runButton, cancelButton, progressBar);
        root.setAlignment(Pos.CENTER);
        root.setSpacing(10);
        
        primaryStage.setScene(new Scene(root, 300, 200));
        primaryStage.setTitle("Background Task");
        primaryStage.show();
    }
}
