import javafx.application.Application;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.geometry.Pos;
import javafx.scene.Node;
import javafx.scene.Scene;
import javafx.scene.control.Label;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import javafx.stage.Stage;

import java.text.DecimalFormat;

public class MyApplication extends Application {
    static final DecimalFormat LABEL_FORMAT = new DecimalFormat("000.00");
    
    Node createKnobAndLabel(double value) {
        Knob knob = new Knob(value);

        Label label = new Label(LABEL_FORMAT.format(knob.getValue()));
        knob.valueProperty().addListener(new ChangeListener<Number>() {
            @Override
            public void changed(ObservableValue<? extends Number> observable,
                                Number oldValue, Number newValue) {
                label.setText(LABEL_FORMAT.format(newValue));
            }
        });

        VBox pane = new VBox(knob, label);
        pane.setAlignment(Pos.CENTER);
        pane.setSpacing(5);
        
        return pane;
    }

    @Override
    public void start(Stage primaryStage) throws Exception {
        HBox root = new HBox();
        root.setAlignment(Pos.CENTER);
        root.getChildren().addAll(
                createKnobAndLabel(0), createKnobAndLabel(50), createKnobAndLabel(100));

        primaryStage.setScene(new Scene(root, 300, 200));
        primaryStage.setTitle("Knob");
        primaryStage.show();
    }

    public static void main(String...args) {
        Application.launch(MyApplication.class, args);
    }
}
