import java.util.Optional;

public class Main {
    void exec(Optional<StringBuilder> osb) {
        // くるんだオブジェクトの単純な取り出し
        StringBuilder sb1 = osb.get();

        // くるんだオブジェクト参照がnullでなければ単純な取り出し
        // nullであれば、orElseの引数に渡したオブジェクトを返す
        StringBuilder sb2 = osb.orElse(new StringBuilder("none"));

        // くるんだオブジェクト参照がnullでなければ単純な取り出し
        // nullであれば、orElseGetの引数に渡した関数の結果を（遅延処理して）返す
        StringBuilder sb3 = osb.orElseGet(() -> new StringBuilder("none")); // 引数はラムダ式（「ラムダ式とストリーム処理」の章を参照）

        // くるんだオブジェクト参照がnullでなければ処理（ラムダ式またはメソッド参照）を実行
        // nullであれば何もしない（リスト4.2のnullチェックをなくせる）
        osb.ifPresent(sb -> {
            sb.insert(0, "[");
            sb.append("]");
        });

        // くるんだオブジェクト参照がnullでなければ関数（ラムダ式またはメソッド参照）を実行して結果を返す
        // nullであれば空Optionalを返す
        Optional<String> os = osb.map(sb -> {
            sb.insert(0, "[");
            sb.append("]");
            return sb.toString();
        });
    }

    public static void main(String... args) {
        Main main = new Main();
        StringBuilder sb = new StringBuilder("foo");
        main.exec(Optional.of(sb));
        System.out.println(sb);
    }
}
