import java.util.Optional;

class Name {
    private final Optional<String> first;
    private final Optional<String> last;
    Name(String first, String last) {
        this.first = Optional.ofNullable(first);
        this.last = Optional.ofNullable(last);
    }
    Optional<String> getFirst() {
        return this.first;
    }
}

class Person {
    private final Optional<Name> name;
    Person(Name name) {
        this.name = Optional.ofNullable(name);
    }
    Optional<Name> getName() {
        return this.name;
    }
}

public class Main {
    // リスト4.4の書き換え（「ラムダ式とストリーム処理」の章で説明するメソッド参照を使っています）
    void showPersonFirstName(Optional<Person> person) {
        person.flatMap(Person::getName)   // getNameメソッドを呼び出し、結果をOptional型で返す
            .flatMap(Name::getFirst)    // getFirstメソッドを呼び出し、結果をOptional型で返す
            .ifPresent(System.out::println);   // 空Optionalでなければprintln処理を呼ぶ
    }

    public static void main(String... args) {
        Main main = new Main();
        Person person = new Person(new Name("Bill", "Joy"));
        main.showPersonFirstName(Optional.of(person));
    }
}
